#include "util.C"
#include "mapping.C"
#include "TH1F.h"
#include "TCanvas.h"
#include "TPad.h"
#include "TLatex.h"
void analyse(int nrun=-1, int iLED=0, int iSlot=15, int iFE=1, int fit=0, char *psFileName=NULL) {
  // Plot all 18 SiPMs connected to one LED
  //
  //  int iFE=3; // front-end
  //  fit=0/1: don't use/use gain fitted histograms
  //           if(fit==1) {
  //                 loads file "run%d_fit.root",nrun
  //                 and retrieves histos: "FE %d chip %d chan %d hist"
  //                 switch merging of bins off
  //           }

  bool logy=true;
  float cutFractionLeft=0.0002;
  float cutFractionRight=0.0002;

  if(nrun==-1) {
     printf("analyse(int nrun=-1, int iLED=0, int iSlot=15, int iFE=1, int fit=0, char *psFileName=NULL)\n");
     printf("            nrun=0 for allChannels.root\n"); 
     return;
  }
 
  if(psFileName!=NULL) {
     if(!strcmp(psFileName,"default")) {
        sprintf(psFileName,"ps/run%d_slot%d_fe%d_led%d.ps",nrun,iSlot,iFE,iLED);
     }
     printf("--- start analyse(%d,%d,%d,%d,%d,\"%s\")\n",nrun,iLED,iSlot,iFE,fit,psFileName);
  }
  else {
     printf("--- start analyse(%d,%d,%d,%d,%d)\n",nrun,iLED,iSlot,iFE,fit);
  }
  char cSerial[4];

  switch(iSlot) {
     case 5:  sprintf(cSerial,"014"); break; // CERN
     case 9:  sprintf(cSerial,"010"); break; // CERN
     case 12: sprintf(cSerial,"001"); break; // DESY test beam	      
     case 15: sprintf(cSerial,"007"); break; // CERN
     default: printf("unknown iSlot %d\n",iSlot); return;
  } 
   
  int Nmergebins;
  if(fit) Nmergebins=1;
  else Nmergebins=4;

  int LED_chip[18], LED_chan[18]; // chip and channel numbers for iLED
  int ierr=mapping(iLED,LED_chip,LED_chan);
  if(!ierr) {
     printf("loaded mapping for LED %d\n",iLED);
     // now LED_chip[] and LED_chan[] contain the 18 SiPMs that are connected
     // to LED iLED
  }
  else {
     printf("loading mapping for LED %d failed\n",iLED);
     return;
  } 


  // retrieve the 18 histograms
  TH1F *histos[18], *horig;
  char filename[255];
  if(nrun==0) {
     sprintf(filename,"data/allChannels_slot%d_fe%d.root",iSlot,iFE);
  }
  else {
    if(fit) sprintf(filename,"data/run%d_slot%d_fe%d_fit.root",nrun,iSlot,iFE);
    else sprintf(filename,"data/run%d_slot%d_fe%d.root",nrun,iSlot,iFE);
  }
  TFile *file = new TFile(filename); 
  printf("opened file %s\n",filename);
  char histoName[255];
  int i;
  for(i=0; i<18; i++) {
    if(fit) {
      sprintf(histoName,"FE %d chip %d chan %d hist",
	      iFE,LED_chip[i],LED_chan[i]);
    }
    else {
      sprintf(histoName,"SER%s, Slot %2d, FE%d, Chip %2d, Chan %2d",
	      cSerial,iSlot,iFE,LED_chip[i],LED_chan[i]);
    }
    horig=(TH1F*) gDirectory->Get(histoName);
    if(horig==NULL) {
       printf("histogram not found %s\n",histoName);
       return;
    }
    histos[i]=(TH1F*) horig->Rebin(Nmergebins,"h1");
  }
  printf("loaded histograms for LED %d, SER%s, Slot %d, FE%d\n",iLED,cSerial,iSlot,iFE);

  /*****************************************
   *
   *   Plotting
   *
   *****************************************/ 
  TCanvas *c1 = new TCanvas("c1", "SiPM spectra per LED",100,0,1024,768);
  c1->SetBorderMode(0);
  
  TPad *c2 = new TPad("c2","The first subpad",0.,0.,1.,.96);
  TLatex heading;
  heading.SetTextAlign(12);
  heading.SetTextSize(0.02);
  heading.SetNDC();
  char headingFileName[255];
  if(Nmergebins>1) {
    if(nrun==0) {
       sprintf(headingFileName,"allChannels.root - LED %d - Slot %d, FE%d - Entries vs. ADC channels - %d bins merged",iLED,iSlot,iFE,Nmergebins);
    }
    else {
       sprintf(headingFileName,"Run %d - LED %d - Slot %d, FE%d - Entries vs. ADC channels - %d bins merged",nrun,iLED,iSlot,iFE,Nmergebins);
    }
  }
  else {
    if(nrun==0) {
       sprintf(headingFileName,"allChannels.root - LED %d - Slot %d, FE%d - Entries vs. ADC channels",iLED,iSlot,iFE);
    }
    else { 
       sprintf(headingFileName,"Run %d - LED %d - Slot %d, FE%d - Entries vs. ADC channels",nrun,iLED,iSlot,iFE);
    }
  }
  heading.DrawLatex(0.02,0.98,headingFileName);
  c1->Draw();
  c2->Draw();
  c2->SetFillColor(0);
  c2->SetBorderMode(0);
  c2->Divide(5,4);
  c2->Update();
  char subPadName[10];
  TPad *c2_sub[20];
  for(i=0; i<20; i++) {
    sprintf(subPadName,"c2_%d",i+1);
    c2_sub[i] = (TPad*) c2->FindObject(subPadName);
    c2_sub[i]->SetBorderMode(0);
    c2_sub[i]->SetLeftMargin(0.08); // % of pad width (default is 10%)
    c2_sub[i]->SetRightMargin(0.005); // % of pad width (default is 10%)
    c2_sub[i]->SetBottomMargin(0.05); // % of pad height (default is 10%)
    if(logy) {
	c2_sub[i]->SetLogy();
    }
  }

  int xbinmin,xbinmax;
  char histotitle[255];
  for(i=0; i<18; i++) {
    c2->cd(i+1);
    
    sprintf(histotitle,"Chip %d, Channel %d",LED_chip[i],LED_chan[i]);
    histos[i]->SetTitle(histotitle);

    // display only populated range
    getMinMaxBin(&xbinmin,&xbinmax,histos[i],cutFractionRight,cutFractionLeft);
    TAxis* xaxis = histos[i]->GetXaxis();
    xaxis->SetRange(xbinmin,xbinmax);
    xaxis->SetTitleSize(0.06);
    xaxis->SetLabelSize(0.045);
    xaxis->SetLabelOffset(0.01);

    TAxis* yaxis = histos[i]->GetYaxis();
    yaxis->SetTitleSize(0.06);
    if(logy) 
       yaxis->SetLabelSize(0.05);
    else
       yaxis->SetLabelSize(0.025);
    histos[i]->Draw();
  }
  if(psFileName!=NULL) c1->Print(psFileName);
  printf("--- end analyse(...)\n");
  //file->Close();
}


void loop(int nrun=-1, int slot=15, int fe=1, int fit=0) {

   if(nrun==-1) {
     printf("loop(int nrun=-1, int slot=15, int fe=1, int fit=0)\n");
     printf("         nrun=0 for allChannels.root\n"); 
     return;
   }
   char psFileName[255];
   char psFileNameProto[255];
   if(fit) {
      if(nrun)
      	sprintf(psFileNameProto,"ps/run%d_slot%d_fe%d_fit_LEDs.ps",nrun,slot,fe);
      else
      	sprintf(psFileNameProto,"ps/allChannels_slot%d_fe%d_fit_LEDs.ps",slot,fe);
   } 
   else {
     if(nrun)
        sprintf(psFileNameProto,"ps/run%d_slot%d_fe%d_LEDs.ps",nrun,slot,fe);
     else
        sprintf(psFileNameProto,"ps/allChannels_slot%d_fe%d_LEDs.ps",slot,fe);
   }

   // Loop over 12 LEDs
   for (int LED=0; LED<12; LED++) {
      if(LED==0) 
         sprintf(psFileName,"%s(",psFileNameProto);
      else if(LED==11) 
         sprintf(psFileName,"%s)",psFileNameProto);
      else
         sprintf(psFileName,"%s",psFileNameProto);
//      printf("loop: calling analyse(%d,%d,%d,%d,%d,\"%s\")\n",nrun,LED,slot,fe,fit,psFileName);
      analyse(nrun,LED,slot,fe,fit,psFileName);
   }
	      
}
