#ifndef AhcReadout_HH
#define AhcReadout_HH

#include <iostream>
#include <fstream>

#include "CrcReadout.hh"
#include "CrcLocationData.hh"
#include "AhcVfeStartUpData.hh"


class AhcReadout : public CrcReadout {

public:
  AhcReadout(unsigned b) : CrcReadout(b,0xac) {
  }

  AhcReadout(unsigned b, unsigned char c) : CrcReadout(b,c) {
  }

  virtual ~AhcReadout() {
  }

  virtual bool record(RcdRecord &r) {
    if(doPrint(r.recordType())) {
      std::cout << "AhcReadout::record()" << std::endl;
      r.RcdHeader::print(std::cout," ") << std::endl;
    }

    SubInserter inserter(r);

    UtlPack tid;
    tid.halfWord(1,SubHeader::ahc);
    tid.byte(2,_location.crateNumber());
    DaqTwoTimer *t(inserter.insert<DaqTwoTimer>(true));
    t->timerId(tid.word());


    if(r.recordType()==RcdHeader::startUp) {
      assert(doAhcVfeStartUpData(r,true));
    }


    if(r.recordType()==RcdHeader::configurationStart) {
      assert(doAhcVfeStartUpData(r,false));

      SubAccessor accessor(r);

      std::vector<const CrcReadoutConfigurationData*>
	bc(accessor.access< CrcReadoutConfigurationData>());
      
      for(unsigned j(0);j<bc.size();j++) {
	if(bc[j]->crateNumber()==_location.crateNumber()) {
	  _config=*(bc[j]);
	  if(doPrint(r.recordType(),1)) _config.print(std::cout," ") << std::endl;
	}
      }

      std::vector<const CrcLocationData<AhcVfeConfigurationData>*>
	v(accessor.access<CrcLocationData<AhcVfeConfigurationData> >());
      
      for(unsigned i(0);i<v.size();i++) {
	if(doPrint(r.recordType(),2))
	  v[i]->print(std::cout," ") << std::endl;
      }

      assert(doAhcVfeConfigurationData(v,r));
    }

    if(r.recordType()==RcdHeader::runEnd) {
      assert(doAhcVfeStartUpData(r,false));



      CrcLocationData<AhcVfeConfigurationData> defaultData(_location);
      defaultData.slotBroadcast(true);
      defaultData.crcComponent(CrcLocation::feBroadcast);

      std::vector<const CrcLocationData<AhcVfeConfigurationData>*> v;
      v.push_back(&defaultData);

      for(unsigned i(0);i<v.size();i++) {
	if(doPrint(r.recordType(),2))
	  v[i]->print(std::cout," ") << std::endl;
      }

      assert(doAhcVfeConfigurationData(v,r));
   }

    if(r.recordType()==RcdHeader::shutDown) {
      assert(doAhcVfeStartUpData(r,false));
    }
 
    bool reply(CrcReadout::record(r));

    // Close off overall timer                                                                                                                
    t->setEndTime();
    if(doPrint(r.recordType())) t->print(std::cout," ") << std::endl;

    return reply;
  }


  bool doAhcVfeStartUpData(RcdRecord &r, bool reset) {
    SubAccessor accessor(r);
    
    std::vector<const CrcLocationData<AhcVfeStartUpData>*>
      v(accessor.access<CrcLocationData<AhcVfeStartUpData> >());
    
    SubInserter inserter(r);

    CrcLocationData<AhcVfeStartUpData> readData;

    AhcVfeConfigurationData defaultData,dummyData;
    if(doPrint(r.recordType(),2)) defaultData.print(std::cout," ") << std::endl;

    for(unsigned i(0);i<v.size();i++) {      
      assert(v[i]->label()==1);
      assert(v[i]->crateNumber()==0xac);
      assert(!v[i]->slotBroadcast());
      assert(v[i]->crcComponent()<=CrcLocation::fe7);

      unsigned s(v[i]->slotNumber());
      assert(s>=0 && s<=21);

      if(_device[s]!=0) {
	assert(_device[s]->writeAhcVfeStartUpData(v[i]->crcComponent(),
						  *(v[i]->data()),
						  *(readData.data()),reset));

	// Repeat at StartUp so as to get actual values in read
	if(reset)
	assert(_device[s]->writeAhcVfeStartUpData(v[i]->crcComponent(),
						  *(v[i]->data()),
						  *(readData.data()),false));
	readData.location(v[i]->location());
	readData.label(2);
	inserter.insert(readData);
	
	if(doPrint(r.recordType(),2)) readData.print(std::cout," ") << std::endl;

	// Load known configuration value on startUp
	if(reset) {
	  assert(_device[s]->writeAhcVfeConfigurationData(v[i]->crcComponent(),
							  defaultData,dummyData));
	}
      }
    }

    return true;
  }

  bool doAhcVfeConfigurationData(std::vector<const CrcLocationData<AhcVfeConfigurationData>*> &v, RcdRecord &r) {

    // Cannot write multiple times as lose previous values
    // Find single value for each slot/FE
    AhcVfeConfigurationData writeData[22][8];

    // First look for slot and FE broadcast
    for(unsigned i(0);i<v.size();i++) {      
      if(v[i]->label()==1) {
	assert(v[i]->crateNumber()==0xac);
	if(v[i]->slotBroadcast() && v[i]->crcComponent()==CrcLocation::feBroadcast) {
	  for(unsigned s(0);s<=21;s++) {
	    for(unsigned f(0);f<8;f++) {
	      writeData[s][f]=*(v[i]->data());
	    }
	  }
	}
      }
    }

    // Look for slot broadcast and not FE broadcast
    for(unsigned i(0);i<v.size();i++) {      
      if(v[i]->label()==1) {
	assert(v[i]->crateNumber()==0xac);
	if(v[i]->slotBroadcast() && v[i]->crcComponent()<=CrcLocation::fe7) {
	  for(unsigned s(0);s<=21;s++) {
	    writeData[s][v[i]->crcComponent()]=*(v[i]->data());
	  }
	}
      }
    }
	  
    // Look for not slot broadcast and FE broadcast
    for(unsigned i(0);i<v.size();i++) {      
      if(v[i]->label()==1) {
	assert(v[i]->crateNumber()==0xac);
	if(!v[i]->slotBroadcast() && v[i]->crcComponent()==CrcLocation::feBroadcast) {
	  unsigned s(v[i]->slotNumber());
	  assert(s>=0 && s<=21);

	  for(unsigned f(0);f<8;f++) {
	    writeData[s][f]=*(v[i]->data());
	  }
	}
      }
    }

    // Look for not slot broadcast and not FE broadcast
    for(unsigned i(0);i<v.size();i++) {
      if(v[i]->label()==1) {
	assert(v[i]->crateNumber()==0xac);
	if(!v[i]->slotBroadcast() && v[i]->crcComponent()<=CrcLocation::fe7) {
	  unsigned s(v[i]->slotNumber());
	  assert(s>=0 && s<=21);

	  writeData[s][v[i]->crcComponent()]=*(v[i]->data());
	}
      }
    }

    // Now load
    SubInserter inserter(r);

    CrcLocationData<AhcVfeConfigurationData> readData;
    readData.crateNumber(0xac);

    if(doPrint(r.recordType(),1)) 
      _config.print(std::cout," ") << std::endl;

    for(unsigned s(2);s<=21;s++) {
      if(_device[s]!=0 && _config.slotEnable(s)) {
	for(unsigned f(0);f<8;f++) {
	  if(_config.slotFeEnable(s,f)) {
	    if(doPrint(r.recordType(),2)) writeData[s][f].print(std::cout," ") << std::endl;

	    assert(_device[s]->writeAhcVfeConfigurationData((CrcLocation::CrcComponent)f,
							    writeData[s][f],
							    *(readData.data())));
	    readData.slotNumber(s);
	    readData.crcComponent((CrcLocation::CrcComponent)f);
	    readData.label(2);
	    inserter.insert(readData);
	    
	    if(doPrint(r.recordType(),2)) readData.print(std::cout," ") << std::endl;
	  }
	}
      }
    }

    return true;
  }


private:
};

#endif
